function IJAWH:SetSmithingResultTooltip(recipeData)
	local _, _, icon = GetSmithingPatternInfo(IJAWH:GetAllCraftingParametersWithoutIteration(recipeData))
	local itemLink = GetSmithingPatternResultLink(IJAWH:GetAllCraftingParametersWithoutIteration(recipeData))
	if itemLink and itemLink ~= "" then
		if IsInGamepadPreferredMode() then
			ZO_GamepadSmithingTopLevelCreationResultTooltipContainerTip:ClearLines()
			ZO_GamepadSmithingTopLevelCreationResultTooltipIcon:SetTexture(icon)
			ZO_GamepadSmithingTopLevelCreationResultTooltipIcon:SetHidden(false)
			ZO_GamepadSmithingTopLevelCreationResultTooltipContainerTip:LayoutItem(itemLink, NOT_EQUIPPED)
		else
			ZO_SmithingTopLevelCreationPanelResultTooltip:ClearLines()		
			ZO_SmithingTopLevelCreationPanelResultTooltipIcon:SetTexture(icon)
			ZO_SmithingTopLevelCreationPanelResultTooltipIcon:SetHidden(false)
			ZO_SmithingTopLevelCreationPanelResultTooltip:SetPendingSmithingItem(IJAWH:GetAllCraftingParametersWithoutIteration(recipeData))
		end
	end
end

local STYLE_INDEX = 0
local STYLE_LIST_INDEX = 0
local function GetItemDataFilterComparator(itemType)
	return function(itemData)
		if itemData.itemType == itemType then
			return true
		end
	end
end

function IJAWH:SetSmithingStyleList()
	local styleList = {}
	local numStyles = self.savedVariables.useRaceStyles and 34 or GetNumValidItemStyles()
	
	local i = 1
	while i <= numStyles  do
		local itemStyleId = GetValidItemStyleId(i)
		local styleName = GetItemStyleName(itemStyleId)
		local stackCount = GetCurrentSmithingStyleItemCount(itemStyleId)
			if stackCount > 0 then
				table.insert(styleList,
					{	
						stackCount = stackCount, 
						styleIndex = itemStyleId,
						styleItem = itemName
					}
				)
			end
		if i == 9 and self.savedVariables.useRaceStyles then 
			i = GetImperialStyleId()
		else
			i = i + 1
		end
	end
	
	table.sort(styleList, function(a,b)return a.stackCount > b.stackCount end)
	
	IJAWH.SmithingStyleList = styleList
end

local function getNextKnownStyleIndex(patternIndex, totalItems)
	while STYLE_INDEX <= GetNumValidItemStyles() do
		local itemStyleId = GetValidItemStyleId(STYLE_INDEX)
		if IsSmithingStyleKnown(itemStyleId, patternIndex) then
			local stackCount = GetCurrentSmithingStyleItemCount(itemStyleId)
			if stackCount >= totalItems then
				return itemStyleId
			end
		end
		STYLE_INDEX = STYLE_INDEX + 1
	end
	return 0
end

local function getNextKnownStyleListIndex(patternIndex, maximum)
	while STYLE_LIST_INDEX <= #IJAWH.SmithingStyleList do
		if IJAWH.SmithingStyleList[STYLE_LIST_INDEX] then
			local styleIndex = IJAWH.SmithingStyleList[STYLE_LIST_INDEX].styleIndex
			if IsSmithingStyleKnown(styleIndex, patternIndex) and IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount >= maximum then
				return
			end
		end
		STYLE_LIST_INDEX = STYLE_LIST_INDEX + 1
    end
	STYLE_LIST_INDEX = 0
    return
end

local function mustUseNewStyle(patternIndex, maximum)
	if not IJAWH.SmithingStyleList[STYLE_LIST_INDEX] then 
		return true
	elseif not IsSmithingStyleKnown(IJAWH.SmithingStyleList[STYLE_LIST_INDEX].styleIndex, patternIndex) then
		return true
	elseif IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount < maximum then
		return true
	end
	return false
end

local function getStyleIndex(maximum, totalItems, patternIndex)
	if IJAWH.savedVariables.useMostStyle then 
		if mustUseNewStyle(patternIndex, maximum) then
			getNextKnownStyleListIndex(patternIndex, maximum)
			if STYLE_LIST_INDEX > 0 then
				IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount = IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount - maximum
			   return IJAWH.SmithingStyleList[STYLE_LIST_INDEX].styleIndex
			else
				IJAWH:AlertQue(SOUNDS.NEGATIVE_CLICK, GetString(SI_TRADESKILLRESULT142))
				return 0	-- "Missing Style Material"
			end
		else
			IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount = IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount - maximum
			return IJAWH.SmithingStyleList[STYLE_LIST_INDEX].styleIndex
		end
	else
		STYLE_INDEX = 1
		local itemStyleId = getNextKnownStyleIndex(patternIndex, totalItems)
		if itemStyleId > 0 then
			return itemStyleId
		else
			IJAWH:AlertQue(SOUNDS.NEGATIVE_CLICK, GetString(SI_TRADESKILLRESULT142))
			return 0	-- "Missing Style Material"
		end
	end
end
	
local function getSmithingMaterialIndex()
	local CraftingInteractionType = GetCraftingInteractionType()

	local skillRank = IJAWH:GetSkillRank(CraftingInteractionType, 1)
	
	if CraftingInteractionType == CRAFTING_TYPE_JEWELRYCRAFTING then
		return skillRank > 0 and select(skillRank , 1, 13, 26, 33, 40) or 1
	else
		return skillRank > 0 and select(skillRank , 1, 8, 13, 18, 23, 26, 29, 32, 34, 40) or 1
	end
end

local function getSmithingPatternIndex(qIndex, lineId, traitIndex, maximum)
	local materialIndex = getSmithingMaterialIndex()
	for patternIndex = 1, GetNumSmithingPatterns() do
		local _,_, materialQuantity = GetSmithingPatternMaterialItemInfo(patternIndex, materialIndex)
		itemLink = GetSmithingPatternResultLink(patternIndex, materialIndex, materialQuantity, GetFirstKnownStyleIndex(), traitIndex, maximum)
		if DoesItemLinkFulfillJournalQuestCondition(itemLink, qIndex, 1, lineId) then
			return patternIndex, materialIndex, materialQuantity, itemLink
		end
	end
end

function IJAWH:UpdateSmithingStyleIndex(maximum, patternIndex, count)
	local styleIndex, totalItems = 0 , 0
	if count then
		totalItems = count
	else
		totalItems = maximum
	end
	
	styleIndex = getStyleIndex(maximum, totalItems, patternIndex)
	return styleIndex
end

function IJAWH:GetSmithingDetails(qIndex, lineId, step, qName, maximum, totalMaterialQuantity, totalItems, conditiona)
	local traitIndex, styleIndex, itemLink, canCraft = 1, 0, nil, true
	totalItems = totalItems + maximum
	local patternIndex, materialIndex, materialQuantity, itemLink = getSmithingPatternIndex(qIndex, lineId, traitIndex, maximum)

	if itemLink ~= nil then	
		local stackCount = GetCurrentSmithingMaterialItemCount(patternIndex, materialIndex)

		-- totalMaterialQuantity is used to predetermine the amount used for crafting all items for the station.
		-- this is used to determine if there is enough material to craft each and all items.
		totalMaterialQuantity = totalMaterialQuantity + materialQuantity

		if totalMaterialQuantity > stackCount then
			IJAWH:AlertQue(SOUNDS.NEGATIVE_CLICK, GetString(SI_TRADESKILLRESULT141))
			canCraft = false	-- "Missing Crafting Material"
		end
		if traitIndex > 1 and totalItems >  GetCurrentSmithingTraitItemCount(traitIndex) then
			IJAWH:AlertQue(SOUNDS.NEGATIVE_CLICK, GetString(SI_TRADESKILLRESULT143))
			canCraft = false	-- "Missing Trait Material"
		end
		if not CheckInventorySpaceSilently(totalItems) then
			IJAWH:AlertQue(SOUNDS.NEGATIVE_CLICK, GetString(SI_ACTIONRESULT3430))
			canCraft = false	-- "You do not have enough inventory space."
		end

		-- The selected style at this point is primarily used to build the initial crafting data
		-- and are not necessarily the ones that will be used for crafting. Style quantity will be dependent on
		-- what order the smithing writs, and items, are crafted.
		styleIndex = getStyleIndex(maximum, totalItems, patternIndex)
		
		-- get selected style item stack count remaining after previous items
		-- 2 determine if there is enough left over to craft all items of the current item. "Craft restoration staff:0/2"
	 	local adjustedStyleItemCount = GetCurrentSmithingStyleItemCount(styleIndex) - (totalItems - maximum)
		
		if styleIndex == 0 then
			styleIndex = GetFirstKnownItemStyleId(patternIndex)
			canCraft = false
--			return totalMaterialQuantity, totalItems, traitIndex, itemLink 
		end

--		if maximum > 1 and IJAWH.SmithingStyleList[STYLE_LIST_INDEX].stackCount < maximum then
		if maximum > 1 and adjustedStyleItemCount < maximum then
			-- for when there is more than one item of the same type (craft 2 oak restoration staffs)
			-- but only have 1 style material of selected type
			-- this is mainly to separate the items while crafting
			for i=1, maximum do
				IJAWH_CurrentWrit.recipeData[#IJAWH_CurrentWrit.recipeData+1] = {
					patternIndex 		= patternIndex,
					materialIndex 		= materialIndex, 
					materialQuantity 	= materialQuantity, 
					styleIndex			= styleIndex,
					traitIndex 			= traitIndex,
					numIterations 		= 1,
					itemLink			= itemLink,
					func 			= function(params)
											return GetCurrentSmithingMaterialItemCount(params.patternIndex, params.materialIndex) >= params.materialQuantity and
												GetCurrentSmithingStyleItemCount(params.styleIndex) >= params.numIterations and
												(params.traitIndex > 1 and GetCurrentSmithingTraitItemCount(params.traitIndex) >= params.numIterations or true)
										end
				}
			end
		else
			IJAWH_CurrentWrit.recipeData[#IJAWH_CurrentWrit.recipeData+1] = {
				patternIndex 		= patternIndex,
				materialIndex 		= materialIndex, 
				materialQuantity 	= materialQuantity, 
				styleIndex			= styleIndex,
				traitIndex 			= traitIndex,
				numIterations 		= maximum,
				itemLink			= itemLink,
				func 			= function(params)
										return GetCurrentSmithingMaterialItemCount(params.patternIndex, params.materialIndex) >= params.materialQuantity and
											GetCurrentSmithingStyleItemCount(params.styleIndex) >= params.numIterations and
											(params.traitIndex > 1 and GetCurrentSmithingTraitItemCount(params.traitIndex) >= params.numIterations or true)
									end
			}
		end
		
		return totalMaterialQuantity, totalItems, traitIndex, itemLink
	end
	return totalMaterialQuantity, totalItems, traitIndex, itemLink
end

function IJAWH:ParseSmithingQuest(qName, qIndex)
	STYLE_LIST_INDEX = 0
	IJAWH:SetSmithingStyleList()
	local totalMaterialQuantity, totalItems = 0, 0
	local step, traitIndex = 0, 0
	IJAWH_CurrentWrit.recipeData = {}
	
	for lineId = 1, GetJournalQuestNumConditions(qIndex,1) do
		local inBank, itemLink, amountRequired = false,'',''
		local condition,current,maximum,_,complete = GetJournalQuestConditionInfo(qIndex,1,lineId)
		if condition ~= '' then
			if IJAWH:Contains(condition, IJAWH:AcquireConditions()) then
				if current ~= maximum then
					amountRequired, itemLink, inBank = IJAWH:GetAcquireItem(qName, lineId, condition, maximum)
				end
			elseif IJAWH:Contains(condition, IJAWH:CraftingConditions()) then
				if current ~= maximum then
					step = step + 1
					totalMaterialQuantity, totalItems, traitIndex, itemLink = self:GetSmithingDetails(qIndex, lineId, step, qName, maximum, totalMaterialQuantity, totalItems, condition)
				end
			end
		end
	end
	if #IJAWH_CurrentWrit.recipeData > 0 then
		IJAWH.writData[IJAWH_WD_INDEX].recipeData = IJAWH_CurrentWrit.recipeData
	else
		IJAWH_CurrentWrit.recipeData = nil
		IJAWH.writData[IJAWH_WD_INDEX] = nil
	end
end